/*
 * libjingle
 * Copyright 2013, Google Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import <AVFoundation/AVFoundation.h>

#import "APPRTCVideoView.h"

#import "TEMProgressView.h"

#define LABEL_HEIGHT 21

@interface APPRTCVideoView () {
    CGSize renderSize;
}

@property(nonatomic, weak) UIActivityIndicatorView* activityView;
@property(nonatomic, weak) UIButton* glassButton;

@property(nonatomic, weak) TEMProgressView* progressView;

@end

@implementation APPRTCVideoView

@synthesize videoOrientation = _videoOrientation;

static void init(APPRTCVideoView *self, UIView *renderView) {
    renderView.frame = self.bounds;
    [self addSubview:renderView];
    renderView.autoresizingMask = UIViewAutoresizingFlexibleHeight |
    UIViewAutoresizingFlexibleWidth;
    renderView.translatesAutoresizingMaskIntoConstraints = YES;
    self.renderView = renderView;
    
    UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, self.bounds.size.width, LABEL_HEIGHT)];
    label.textColor = [UIColor blueColor];
    label.textAlignment = NSTextAlignmentLeft;
    label.numberOfLines = 1;
    label.autoresizingMask = UIViewAutoresizingFlexibleWidth;
    label.translatesAutoresizingMaskIntoConstraints = YES;
    [self addSubview:label];
    self.lblTitle = label;
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.frame = CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height);
    button.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    button.translatesAutoresizingMaskIntoConstraints = YES;
    [button addTarget:self action:@selector(iAmTapped:) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:button];
    self.glassButton = button;
}

- (id)initWithFrame:(CGRect)frame videoView:(UIView*)videoView {
    self = [super initWithFrame:frame];
    if (self) {
        init(self, videoView);
        _isEnable = TRUE;
        renderSize = CGSizeZero;
    }
    return self;
}

- (void)layoutSubviews {
    [super layoutSubviews];
    if (_renderView && (renderSize.height > 0 && renderSize.width > 0)) {
        CGSize defaultAspectRatio = CGSizeMake(4, 3);
        CGSize aspectRatio = CGSizeEqualToSize(renderSize, CGSizeZero) ? defaultAspectRatio : renderSize;
        CGRect videoFrame = AVMakeRectWithAspectRatioInsideRect(aspectRatio, self.bounds);
        self.renderView.frame = self.glassButton.frame = videoFrame;
        self.lblTitle.frame = CGRectMake(self.renderView.frame.origin.x, self.renderView.frame.origin.y, self.renderView.frame.size.width, LABEL_HEIGHT);
        self.progressView.frame = CGRectMake(self.lblTitle.frame.origin.x, CGRectGetMaxY(self.lblTitle.frame), self.lblTitle.frame.size.width, self.lblTitle.frame.size.height);
    }
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)addProgressView {
    // Listen to the 'SKYLINKFileProgress' Notification.
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(gotProgress:) name:@"SKYLINKFileProgress" object:nil];
    TEMProgressView *progressView = [[TEMProgressView alloc] initWithFrame:CGRectMake(self.lblTitle.frame.origin.x, CGRectGetMaxY(self.lblTitle.frame), self.lblTitle.frame.size.width, self.lblTitle.frame.size.height)];
    [self insertSubview:progressView belowSubview:self.glassButton];
    self.progressView = progressView;
}

- (void)layoutMySubviews:(CGSize)size {
    renderSize = CGSizeEqualToSize(size, CGSizeZero) ? renderSize : size;
    [self setNeedsLayout];
}

- (void)removeProgressView {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self.progressView removeFromSuperview];
    self.progressView = nil;
}

#pragma mark - Properties

- (void)setIsEnable:(BOOL)enable {
    if (_isEnable != enable) {
        _isEnable = enable;
        if (enable) {
            self.glassButton.alpha = 1.0;
            self.glassButton.backgroundColor = [UIColor clearColor];
        } else {
            self.glassButton.alpha = 0.5;
            self.glassButton.backgroundColor = [UIColor lightGrayColor];
        }
    }
}

- (void)setIsRemote:(BOOL)isRemote {
    if (_isRemote != isRemote) {
        _isRemote = isRemote;
        if (isRemote)
            self.renderView.transform = CGAffineTransformMakeScale(-1, 1);
    }
}

- (NSString*)title {
    return self.lblTitle.text;
}

- (void)setTitle:(NSString *)_title {
    self.lblTitle.text = _title;
}

#pragma mark - IBAction

- (IBAction)iAmTapped:(id)sender {
    if ([self.delegate respondsToSelector:@selector(iAmTapped:)])
        [self.delegate iAmTapped:self];
}

#pragma mark - Private

/**
 @discussion Update progress bar upon getting 'SKYLINKFileProgress' Notification.
 */
- (void)gotProgress:(NSNotification*)notification {
    if ([self.ID caseInsensitiveCompare:[notification.userInfo objectForKey:@"peerId"]] == NSOrderedSame)
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.progressView setPercentage:[[notification.userInfo objectForKey:@"percentage"] floatValue]];
        });
}

@end
